#include "channel.h"
#include <climits>


/**
 * @brief Channel::Channel creates a new buffer for one channel.
 */
Channel::Channel() {
    _pointBuffer = new QVector<QPointF>(BUFFER_SIZE);  // Create new buffer for the sampled data.
    _pointBuffer->reserve(BUFFER_SIZE);
}

/**
 * @brief Channel::addPoint adds a new point to the cyclic buffer.
 *
 * @param newpoint The new point to add to the cyclic buffer.
 */
void Channel::addPoint( QPointF newpoint) {
    _pointBuffer->replace( _current_write, newpoint);
    incBufferIndex( _current_write);

    // If we have just overwritten the earliest non-read element, inc the read
    // index to remove it.
    if( _current_write == _current_read) {
        incBufferIndex( _current_read);
    }
}

/**
 * @brief getCurrentBuffer copies the current point buffer to a vector.
 *
 * @param trigger is an optional trigger to find a start point.
 * @param npoints The max number of points or -1 if all the points are wanted.
 *
 * @return The circular buffer as a sorted vector.
 */
QVector<QPointF> Channel::getCurrentBuffer( Trigger *trigger, int nPoints) {

    int cur_index = _current_read;

    if( trigger != NULL) {
        // ToDo: find matching start point
    }

    if( nPoints == -1) {
        nPoints = INT_MAX;
    }

    // This is really slow! Replace it with something better!
    QVector<QPointF> result;

    int curPointIndex = 0;
    while( cur_index != _current_write && result.count() < nPoints) {
        //result.append( _pointBuffer->at( cur_index));
        QPointF curPoint = readPoint();
        curPoint.setX(curPointIndex++);
        result.append(curPoint);
        incBufferIndex( cur_index);
    }

    return result;
}

/**
 * @brief Channel::incBufferIndex increments an index for the circular buffer.
 *
 * @param index The index to increment.
 */
void inline Channel::incBufferIndex( int &index) {

    if( ++index == BUFFER_SIZE) {
        index = 0;
    }
}

/**
 * @brief Channel::readPoint reads a point from the channel buffer or returns a (0,0) point
 * if there are no more points available in the buffer.
 *
 * @return A point from the buffer or (0,0) if there are no more points.
 */
QPointF Channel::readPoint() {
    if( _current_read != _current_write) {
        QPointF result = _pointBuffer->at(_current_read);
        incBufferIndex( _current_read);
        return result;
    } else {
        return NULL_POINT;
    }
}
