#ifndef CHANNEL_H
#define CHANNEL_H

#define BUFFER_SIZE (1024*1024)  // buffer size of the point buffer.

#include "oscilloscope/trigger.h"
#include <qpoint.h>
#include <qvector.h>


/**
 * @brief The Channel class holds the sampled data of one oscilloscope channel.
 */
class Channel
{
private:
    QPointF NULL_POINT = QPointF( 0.0, 0.0);
    QVector<QPointF> *_pointBuffer; // A circular buffer for the sampled data.
    int _current_read = 0; // current read index.
    int _current_write = 0; // current write index.
public:
    Channel();
    void addPoint( QPointF newpoint);  // Add a new point to the channel buffer.
    QPointF readPoint();  // Read a point from the channel buffer or a null point, if there are no more points.
    QVector<QPointF> getCurrentBuffer( Trigger *trigger, int nPoints);
    void inline incBufferIndex( int &index);
};

#endif // CHANNEL_H
