/**
  * Interface for usb oscilloscope hantek 6022be
  *
  * @author Andreas Rueckert <arueckert67@t-online.de>
  *
  * This is a port of this code: https://github.com/rpcope1/Hantek6022API
  *
  */


#ifndef SCOPEINTERFACE_H
#define SCOPEINTERFACE_H

#include <libusb-1.0/libusb.h>
#include "scopedata.h"
#include <string>
#include <vector>

using namespace std;


// Status of the scope connectivity
enum scopeStatus { not_found, found, flashed, is_setup, capturing };


struct UsbFirmwarePacket
{
    u_int64_t length;
    unsigned int value;
    string data;
};

/**
 * The interface to the hantek 6022be scope.
 */
class ScopeInterface {

private:
    int _nActiveChannels = 0;
    libusb_context       *_context;      // A libusb session
    libusb_device_handle *_scope_device_handle; // The used scope.
    unsigned char *_read_buffer;  // Permanent buffer for faster reading.
    scopeStatus _scopeStatus = not_found;

public:
    ScopeInterface();
    ~ScopeInterface();
    bool flash_firmware();
    scopeStatus getScopeStatus();
    void list_usb_devices();
    bool open_handle();
    bool read_data( int nChannels, int length, ScopeData *result);
    bool setup();
    bool set_sample_rate( unsigned char rate_code);
    bool set_voltage_range( int channel, unsigned char range_code);
    bool start_capture();
    bool stop_capture();

private:
    vector<UsbFirmwarePacket> read_firmware(string filename);
    vector<UsbFirmwarePacket> read_firmware_csv(string filename);
    vector<UsbFirmwarePacket> read_firmware_ihex( string filename);
};

#endif // SCOPEINTERFACE_H
